//
mod init {
    use crate::Result;

    #[test]
    fn should_not_update_file_if_it_exists() -> Result<()> {
        let fs = kxio::fs::temp()?;
        let file = fs.base().join(".git-next.toml");
        fs.file(&file).write("contents")?;

        crate::init::run(&fs)?;

        assert_eq!(
            fs.file(&file).reader()?.to_string(),
            "contents",
            "The file has been changed"
        );

        Ok(())
    }

    #[test]
    fn should_create_default_file_if_not_exists() -> Result<()> {
        let fs = kxio::fs::temp()?;

        crate::init::run(&fs)?;

        let file = fs.base().join(".git-next.toml");

        assert!(fs.path(&file).exists()?, "The file has not been created");

        assert_eq!(
            fs.file(&file).reader()?.to_string(),
            include_str!("../default.toml"),
            "The file does not match the default template"
        );

        Ok(())
    }
}

mod errors {
    // all errors should be Send
    const fn is_send<T: Send>() {}

    #[test]
    const fn core_git_file_error_is_send() {
        is_send::<crate::core::git::file::Error>();
    }

    #[test]
    const fn core_git_repository_error_is_send() {
        is_send::<crate::core::git::repository::Error>();
    }

    #[test]
    const fn core_git_forge_webhook_error_is_send() {
        is_send::<crate::core::git::forge::webhook::Error>();
    }

    #[test]
    const fn core_git_validation_remotes_error_is_send() {
        is_send::<crate::core::git::validation::remotes::Error>();
    }
}
